function limit = wsize(Gre, Gim)

% function limit = wsize(Gre, Gim) determine the window size
% where Gre is the real part of the real axis roots
% and   Gim is the imaginary part of the real axis roots

% 97 June 3
% 2004 June 19

if nargin == 0
    display('USAGE: limit = wsize(Gre,Gim)')
    return
elseif nargin < 1 | nargin > 2
    error('Incorrect number of input arguments!')
end
Re_Max = max(Gre);
Re_Min = min(Gre);
Im_Max = max(abs(Gim));
Im_Min = -Im_Max;
xdiv = (Re_Max - Re_Min)/20;                % x-axis interval
ydiv = (Im_Max - Im_Min)/20;                % y-axis interval
xmin = Re_Min - xdiv;                       % x-axis starting point
xmax = Re_Max + xdiv;                       % x-axis ending point
ymin = Im_Min - ydiv;                       % y-axis starting point
ymax = Im_Max + ydiv;                       % y-axis ending point
limit = [xmin, xmax, ymin, ymax];

if all(Gre == 0),
   limit = [-1, 1, ymin, ymax];
elseif all(Gim == 0),
   limit = [xmin, xmax, -1, 1];
end

