% script for testing the UNB Popov absolute stability routine
% JH Taylor - 2 July 1998
clear all; close all; clc

disp('  ')
disp('Demonstration of the UNB single-variable Popov absolute stability routine.')
disp('  ')
disp('After each example there will be a pause; you may hit any key to continue')
disp('   or hit Ctrl-C if you wish to quit.  Please note that the examples show')
disp('   many interesting situations and options.')
disp('  ')
disp('Notes: ')
disp('1. If F_min ~= 0 the Popov plot drawn is actually that of W_bar = ')
disp('   W/(1+F_min*W) where W is given by the user as (num,den) or (A,B,C,D).')
disp('2. If F_min = NaN (you seek F_min for a given F_max) the Popov plot drawn is')
disp('   actually that of W_bar = -W/(1+F_max*W) where W is given by the user.')
disp('3. This routine is not guaranteed foolproof - finding the optimal Popov line')
disp('   is tricky and we had to use heuristics - but testing has been extensive.')
disp('4. The labelling on the Popov plots (num = ... etc.) is not automatic, but')
disp('   added to document the example. ')
disp('  ')
disp('Fig. 1: Fourth-order Stable Plant with lightly-damped poles:')
disp('  ')
num = [ 1 1 ];
den = [ 1 2 25 3 1 ];
poles = roots(den)
figure;[Fmin,Fmax] = popov(num,den,0);
title('\bf\itPopov Criterion Study of a Stable Plant, F_{min} = 0')
text(0.1,-0.12,'\bf\itnum = [ 1  1 ]');
text(0.1,-0.15,'\bf\itden = [ 1  2  25  3  1 ]');
text(0.1,-0.18,['\bf\itF_{min} = ',num2str(Fmin)])
text(0.1,-0.21,['\Rightarrow  \bf\itF_{max} = ',num2str(Fmax)])
pause
disp('  ')
disp('Fig. 2: Same Stable Plant, non-zero Fmin:')
disp('  ')
figure;[Fmin,Fmax] = popov(num,den,2.5);
title('\bf\itPopov Criterion Study of a Stable Plant, F_{min} = 2.5')
text(0.07,-0.06,'\bf\itnum = [ 1  1 ]');
text(0.07,-0.09,'\bf\itden = [ 1  2  25  3  1 ]');
text(0.07,-0.12,['\bf\itF_{min} = ',num2str(Fmin)])
text(0.07,-0.15,['\Rightarrow  \bf\itF_{max} = ',num2str(Fmax)])
pause
disp('  ')
disp('Fig. 3: Nyquist Criterion Study of an Unstable Plant:')
disp('        Note: this is generated by our enhanced routine NEWNYQ')
disp('  ')
num = [ 1 2 ];
den = [ 1 -4 -5 ];
newnyq(num,den);
text(-0.15,0.04,'\bf\itnum = [ 1  2 ]');
text(-0.15,0.025,'\bf\itden = [ 1  -4 -5 ]');
text(-0.15,-0.025,'\bf\itNyquist stability range:');
text(-0.15,-0.04,'\bf\it4 < K < \infty ');
pause
disp('  ')
disp('Fig. 4: Popov Criterion Study of an Unstable Plant - Fmin')
disp('        is chosen in the Nyquist range ( 5 in (4, infinity) ):')
disp('  ')
figure;[fmin,fmax] = popov(num,den,5);
title('\bf\itPopov Study of an Unstable Plant ')
text(0.5,-1.0,'\bf\itnum = [ 1 2 ]');
text(0.5,-1.2,'\bf\itden = [ 1 -4 -5 ]');
text(0.5,-1.4,['\bf\itF_{min} = ',num2str(fmin)])
text(0.5,-1.6,['\Rightarrow  \bf\itF_{max} = ',num2str(fmax)])
pause
disp('  ')
disp('Fig. 5: Second Popov Criterion Study of an Unstable Plant -')
disp('        Fmax is provided and Fmin sought (popov(num,den,NaN,50) ):')
disp('  ')
figure;[fmin,fmax] = popov(num,den,NaN,50);
title('\bf\itSecond Popov Study of an Unstable Plant - F_{min} Sought')
text(-.02,.9,'\bf\itnum = [ 1 2 ]');
text(-.02,.8,'\bf\itden = [ 1 -4 -5 ]');
text(-.02,.7,['\bf\itF_{max} = ',num2str(fmax)])
text(-.02,.6,['\Rightarrow  \bf\itF_{min} = ',num2str(fmin)])
disp('  ')
disp('Nota Bene! This Popov plot seems degenerate due to the ')
disp('  quite large value of Fmax provided.')
disp('  ')
pause
% more cases
disp('  ')
disp('Fig. 6: Popov Criterion Study, Popov Line with Negative Slope:')
disp('  ')
% gives a Popov line w negative slope; Aizerman OK...
n1 = [1000]; d1 = poly([-10 -10 -1+20j -1-20j]);
figure;[fmin,fmax] = popov(n1,d1,0);
title('\bf\itPopov Line with Negative Slope')
text(-.035,.5,'\bf\itnum = [ 1000 ]');
text(-.035,.4,'\bf\itden = poly([-10 -10 -1+20j -1-20j])');
text(-.035,.3,['\bf\itF_{min} = ',num2str(fmin)])
text(-.035,.2,['\Rightarrow  \bf\itF_{max} = ',num2str(fmax)])
pause
disp('  ')
disp('Fig. 7: Popov Criterion Study, Plant with Nice Loops')
disp('  ')
n2 = 10000*poly([-1+18j -1-18j]);
d2 = poly([-10 -10 -1+20j -1-20j -15+25j -15-25j]);
figure; [fmin,fmax] = popov(n2,d2,0);
title('\bf\itPopov Study, Lightly Damped Poles & Zeros')
text(.008,-.5,'\bf\itnum = 10000*poly([-1+18j -1-18j])');
text(.008,-.6,'\bf\itden = poly([-10 -10 -1+20j -1-20j -15+25j -15-25j])');
text(.008,-.7,['\bf\itF_{min} = ',num2str(fmin)])
text(.008,-.8,['\Rightarrow  \bf\itF_{max} = ',num2str(fmax)])
pause
% gives a Popov line w negative slope, hits above & below
disp('  ')
disp('Fig. 8: Popov Line with Negative Slope, Tangent Twice')
disp('  ')
n4 = [1.e6]; d4 = poly([-30 -30 -10+200j -10-200j]);
figure; [fmin,fmax] = popov(n4,d4,30); % no problem...
title('\\bf\itPopov Line with Negative Slope Tangent Above & Below Real Axis')
text(.003,1,'\bf\itnum = [ 1.e6 ])');
text(.003,.8,'\bf\itden = poly([ -30 -30 -10+200j -10-200j ])');
text(.003,.6,['\bf\itF_{min} = ',num2str(fmin)])
text(.003,.4,['\Rightarrow  \bf\itF_{max} = ',num2str(fmax)])
pause
disp('  ')
disp('Fig. 9: Plant with Many Lightly-damped Poles and Zeros')
disp('  ')
n5=poly([-6+34i -6-34i -1.7+9i -1.7-9i]);
d5=poly([-0.001+20i -0.001-20i -0.9+3i -0.9-3i ...
 -2 -0.2+0.5i -0.2-0.5i]);
%% n5=poly([ n5(1) n5(2) n5(3) n5(4)]);
%% d5=poly([ d5(1) d5(2) d5(3) d5(4) d5(5) d5(6) d5(7)]);
figure; [fmin,fmax] = popov(n5,d5,NaN,0.026);
title('\bf\itPopov Plot with Sharp Kinks, Line Tangent Above & Below Real Axis')
text(-12,46,'\bf\itnum = poly([-6+34i -6-34i -1.7+9i -1.7-9i])');
text(-12,40,'\bf\itden = poly([-0.001+20i -0.001-20i -0.9+3i -0.9-3i ... ');
text(-12,36,'           \bf\it-2 -0.2+0.5i -0.2-0.5i])');
text(-12,30,['\bf\itF_{max} = ',num2str(fmax)])
text(-12,24,['\Rightarrow  \bf\itF_{min} = ',num2str(fmin)])
disp('End of demonstration - whew! ')
